#include "Python.h"
#include "tabe.h"

#define UNLESS(E) if(!(E))

static PyObject *
tabelib_sparse_tostr(PyObject *modinfo, PyObject *args)
{
    char *input_str, *output_str = NULL, *tsidb_str = NULL;
    PyObject *parsed_str;

	struct ChuInfo *chu;
	struct TsiDB *tdb;
    /* char buf[1000]; */
    char *tmp_str;
    char *tmp_ptr;
    int i;
    int j;
    
    UNLESS(PyArg_ParseTuple(args,"s|s", &input_str, &tsidb_str)) return NULL;

    tmp_str = malloc(strlen(input_str) + 1);
    output_str = malloc(strlen(input_str)*2 + 1);
    *output_str = 0x0;
    memcpy(tmp_str, input_str, strlen(input_str)+1);
    /* clear character: "\r" and "\n" */
    while ( (tmp_ptr=strchr(tmp_str, 13)) != NULL ) *tmp_ptr=' ';
    while ( (tmp_ptr=strchr(tmp_str, 10)) != NULL ) *tmp_ptr=' ';

    chu = malloc(sizeof(struct ChuInfo));
    memset(chu, 0, sizeof(struct ChuInfo));
    chu->chu = tmp_str;

    if (tsidb_str) 
    {
		tdb = tabeTsiDBOpen(DB_TYPE_DB, tsidb_str, 0);
    }
    else
    {
    	tdb = tabeTsiDBOpen(DB_TYPE_DB, "tsi.db", 0);
    }
/*
    while (1) {
      if (!fgets(buf, 1000, fp)) {
        break;
      }
*/
      tabeChuInfoToChunkInfo(chu);

      for (i = 0; i < chu->num_chunk; i++) {
        printf("%s -> ", (chu->chunk[i]).chunk);
        tabeChunkSegmentationComplex(tdb, chu->chunk+i);
        for (j = 0; j < chu->chunk[i].num_tsi; j++) {
          printf("[%s] ", chu->chunk[i].tsi[j].tsi);
          strcat(output_str, chu->chunk[i].tsi[j].tsi);
          strcat(output_str, " ");
        }
        printf("\n");
      }
/*
    }
*/
    tdb->Close(tdb);
    free(tmp_str);
    free(chu);

    UNLESS(parsed_str = PyString_FromString(output_str)) return NULL;
    free(output_str);
    
    return parsed_str;
}

static struct PyMethodDef tabelib_module_methods[] = {
    { "sparse_tostr", (PyCFunction)tabelib_sparse_tostr, METH_VARARGS,
      "sparse_tostr(input_str [,tsidb_str]) -- Break the given string into words, with a format determined by the format_str",
    },
    { NULL, NULL }		/* sentinel */
};

static char tabelib_module_documentation[] = 
"Use TabeLib library to parse source strings into sequences of words\n"
"\n"
"for use with the Catalog\n"
"\n"
"$Id: tabelibmodule.c,v 1.1 2001/12/21 02:53:00 tcchou Exp $\n"
;


void
inittabelib() 
{
  PyObject *m, *d;
  char *rev="$Revision: 1.1 $";
  
  /* Create the module and add the functions */
  m = Py_InitModule4("tabelib", tabelib_module_methods,
                     tabelib_module_documentation,
                     (PyObject*)NULL,PYTHON_API_VERSION);
  
  /* Add some symbolic constants to the module */
  d = PyModule_GetDict(m);
  PyDict_SetItemString(d, "__version__",
		       PyString_FromStringAndSize(rev+11,strlen(rev+11)-2));

  if (PyErr_Occurred()) Py_FatalError("can't initialize module tabelib");
}
